package GCPlugins::GCbooks::GCAmazon;

###################################################
#
#  Copyright 2005-2009 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
#
###################################################

use strict;
use utf8;

use GCPlugins::GCbooks::GCbooksCommon;

{
    package GCPlugins::GCbooks::GCPluginAmazon;
    
    use base qw(GCPlugins::GCbooks::GCbooksPluginsBase);
    use XML::Simple;
    use LWP::Simple qw($ua);
    use Encode;
    use HTML::Entities;
    use GCUtils;

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
	
        $self->{inside}->{$tagname}++;

        if ($self->{parsingList})
        {
            if ($tagname eq 'div' && $attr->{class}  eq 's-item-container')
            {
                $self->{isSponsored} = 0;
            }	    
            elsif ($self->{isSponsored} eq 1)
            {
                return;
            }
	    # Capture URL of book
	    elsif (($self->{isUrl} == 1) && ($tagname eq 'a')  && ($attr->{class} =~ m/detail-page/))
	    {
                 $self->{itemIdx}++;
                 my $rootURL = '';
                 $rootURL = 'http://'.$self->baseWWWamazonUrl if ($attr->{href} =~ m/^\//);
                 $self->{itemsList}[$self->{itemIdx}]->{url} = $rootURL.$attr->{href};
                 $self->{itemsList}[$self->{itemIdx}]->{title} = $attr->{title};
	         $self->{isUrl} = 0 ;
		 $self->{isTitle} = 1 ;
		 return;
	    }	

	    # Identify beginning of new book (next text is title)
	    elsif (($tagname eq 'li') && ($attr->{id} =~ /result_[0-9]+/ ))
	    {
		 # Create new entry
                 #$self->{itemIdx}++;
	         $self->{isUrl} = 1 ;
		 $self->{isAuthor} = 0 ;
		 return ;
	    }
	    elsif ($tagname eq 'h5')
	    {
	        $self->{isSponsored} = 1;
	    }
	    elsif ($self->{isPublication} && $tagname eq 'span' && $attr->{class} =~ /a-color-secondary/)
	    {
	        $self->{isPublication} = 2;
	    }
        }
        else
        {
 	    # Detection of book themes
	    if ($self->{isTheme} == 0 && $tagname eq 'ul' && $attr->{class} eq "zg_hrsr")
	    {
                $self->{isTheme} = 1 ;
                return ;
	    }		

	    # Detection of book page count
	    elsif ($self->{isPage} == 0 && $tagname eq 'td' && $attr->{class} eq "bucket")
	    {
                $self->{isPage} = 1 ;
                return ;
	    }	

 	    # Detection of authors
	    elsif ($tagname eq 'span' && $attr->{class} =~ m/author/i)
	    {
                $self->{isAuthor} = 1;
                return ;
	    }

	    elsif ($tagname eq 'span' && $self->{isAuthor} eq 2 && $attr->{class} eq 'contribution')
	    {
		$self->{isAuthor} = 3;
	    }
	    
	    # Capture of image
	    elsif ($tagname eq 'img' && ($attr->{id} =~ m/imgBlkFront/i || $attr->{id} =~ m/landingImage/i))
	    {
                #$attr->{adress} =~ /http.*?\.jpg/;
		#$attr->{adress} =~ s|https://images-na.ssl-images-amazon.com/images/I/|http://z2-ec2.images-amazon.com/images/I/|;
		#$self->{curInfo}->{cover} = $attr->{adress};
		$self->{curInfo}->{cover} = $attr->{src};
                return ;
	    }

	    # Detection of book description
	    elsif ($self->{isDescription} == 0 && $tagname eq 'script' && $attr->{id} eq "bookDesc_override_CSS")
	    {
                $self->{isDescription} = 1 ;
                return ;
	    }
	    elsif (($self->{isDescription} == 1) && ($tagname eq 'div'))
	    {
                $self->{isDescription} = 2 ;
                return ;
	    }

	    # Detection title
	    elsif (($tagname eq 'span') && ($attr->{id} =~ m/productTitle/i))
	    {
                $self->{isTitle} = 1 ;
                return ;
	    }
        }
    }

    sub end
    {
        my ($self, $tagname) = @_;

        $self->{inside}->{$tagname}--;

        if ($self->{parsingList})
        {
	    # Identify end of comments
	    if ($self->{isAuthor} eq 2 && $tagname eq 'div')
	    {
	        $self->{isAuthor} = 0;
	    }
	    elsif ($self->{isPublication} > 0 && $tagname eq 'div')
	    {
	        $self->{isPublication} = 0 ;
	        $self->{isAuthor} = 1 ;
	    }
	    elsif ($self->{isTitle} eq 1 && $tagname eq 'a') 
	    {
                $self->{isTitle} = 0 ;
                $self->{isPublication} = 1 ;
	    }	    
        }
        else
        {
	    # Finishing themes analysis
	    if (($self->{isTheme} != 0) && ($tagname eq 'li'))
	    {
                $self->{isTheme} = 0 ;
                return ;
	    }

	    # Finishing description analysis
	    if (($self->{isDescription} != 0) && ($tagname eq 'div'))
	    {
                $self->{isDescription} = 0 ;
                return ;
	    }		    

	    # Finishing publication analysis
	    if (($self->{isPublication} != 0) && ($tagname eq 'li'))
	    {
                $self->{isPublication} = 0 ;
                return ;
	    }		    
        }
    }

    sub text
    {
        my ($self, $origtext) = @_;


        if ($self->{parsingList})
        {
	    # Remove blanks before and after string
            $origtext =~ s/^\s+//;
            $origtext =~ s/\s+$//g;

	    if (($self->{isPublication} == 2) && ($origtext ne ''))
	    {
	        if ($origtext ne $self->{translations}->{by})
	        {
                    $self->{itemsList}[$self->{itemIdx}]->{publication} = $origtext;
                    $self->{isAuthor} = 1;
	        }
	        else 
	        { 
	            $self->{isAuthor} = 2;
	        }
                return ;
	    }	
   
	    # Testing if author
	    elsif (($self->{isAuthor} == 1) && ($origtext ne ''))
	    {
                if ($origtext eq $self->{translations}->{by})
                {
                    $self->{isAuthor} = 2 ;
                }
                else
                {
                    $self->{isAuthor} = 0 ;
                }
	    }

	    # Capture of authors
	    elsif (($self->{isAuthor} == 2) && ($origtext ne ''))
	    {
                if ($self->{itemsList}[$self->{itemIdx}]->{authors} eq '')
                {
                    $self->{itemsList}[$self->{itemIdx}]->{authors} = $origtext;
		}
		else
		{
		   $self->{itemsList}[$self->{itemIdx}]->{authors} .= " " . $origtext;
		}
		return;
	    }	

        }
        else
        {
	    # Remove blanks before and after string
            $origtext =~ s/^\s+//;
            $origtext =~ s/\s+$//g;

	    # Capture of title
	    if (($self->{isTitle} == 1) && ($origtext ne ''))
	    {
                $self->{isTitle} = 0 ;
                $self->{curInfo}->{title} = $origtext;
                return ;
	    }

	    # Capture of page number
	    elsif ($self->{isPage} == 1 && $origtext  =~ /^[0-9]+/ && index($origtext,$self->{translations}->{pages}) eq 0)
	    {
		$origtext =~ s/ .*//;
		$self->{curInfo}->{pages} = $origtext;
		$self->{isPage} = 0 ;
                return ;
	    }

	    # Capture of editor and publication date
	    elsif ($self->{isEditor} == 0 && index($origtext,$self->{translations}->{publisher}) eq 0)
	    {
                $self->{isEditor} = 1 ;
		$self->{isPublication} = 1;
                return ;
	    }
	    elsif (($self->{isEditor} == 1 || $self->{isPublication} == 1) && ($origtext ne ''))
	    {
		my @array = split('\(',$origtext);
		# le champ contenant l'éditeur et la date de parution peut contenir plusieurs parenthèses
		# on suppose que la dernière contient la date
		my $myDate = $array[$#array];
		$array[0] =~ s/^\s+//;
		$array[0] =~ s/\s+$//g;
		$array[0] =~ s/\;//g;
		$myDate =~ s/\)//g;
		$myDate =~ s/^\s+//;
		$myDate =~ s/\s+$//g;
                $self->{curInfo}->{publisher} = $array[0] if ($self->{isEditor} == 1);
		$self->{isEditor} = 0 ;

		if ($self->{isPublication} == 1)
		{
		    eval {
			my $myPattern = "%d %B %Y";
			if ($self->getLang() =~ m/EN/)
			{
			    $myPattern = "%b. %d %Y";
			}
			my $dt1 = new DateTime::Format::Strptime(
			    pattern     => $myPattern,
			    locale      => $self->getLang(),
			    on_error    => 'undef');
			my $dt2 = $dt1->parse_datetime($myDate);
			my $dt3 = new DateTime::Format::Strptime(
			    pattern     => "%d/%m/%Y",
			    locale      => "EN",
			    on_error    => 'croak');
			$self->{curInfo}->{publication} = $dt3->format_datetime($dt2);
		    };
		    $self->{curInfo}->{publication} = $myDate if (! $self->{curInfo}->{publication});
			
		};
                return ;
	    }

	    # Capture of language
	    elsif (($self->{isLanguage} == 0) && index($origtext,$self->{translations}->{language}) eq 0)
	    {
                $self->{isLanguage} = 1 ;
                return ;
	    }
	    elsif (($self->{isLanguage} == 1) && ($origtext ne ''))
	    {
                $self->{curInfo}->{language} = $origtext;
                $self->{isLanguage} = 0 ;
                return ;
	    }

	    # Capture of ISBN
	    elsif (($self->{isISBN} == 0) && index($origtext,$self->{translations}->{isbn}) eq 0)
	    {
                $self->{isISBN} =1 ;
                return ;
	    }
	    elsif (($self->{isISBN} == 1) && ($origtext ne ''))
	    {
		$origtext =~ s|-||gi;
                $self->{curInfo}->{isbn} = $origtext;
		$self->{isISBN} = 0 ;
                return ;
	    }

	    # Capture of series
	    elsif (($self->{isSerie} == 0) && index($origtext,$self->{translations}->{series}) eq 0)
	    {
                $self->{isSerie} =1 ;
                return ;
	    }
	    elsif (($self->{isSerie} == 1) && ($origtext ne ''))
	    {
                $self->{curInfo}->{serie} = $origtext;
		$self->{isSerie} = 0 ;
                return ;
	    }

	    # Capture of book dimensions
	    elsif (($self->{isSize} == 0) && index($origtext,$self->{translations}->{dimensions}) > -1)
	    {
                $self->{isSize} = 1 ;
                return ;
	    }
	    elsif (($self->{isSize} == 1) && ($origtext ne ''))
	    {
                $self->{curInfo}->{format} = $origtext;
                $self->{isSize} = 0 ;
                return ;
	    }


	    # Detection of themes
	    elsif (($origtext eq '>') && ($self->{isTheme} == 1))
	    {
                $self->{isTheme} = 2 ;
                return ;
	    }

	    # Capture of themes
	    elsif (($self->{isTheme} == 2) && ($origtext ne ''))
            {
                if ($self->{curInfo}->{genre} eq '')
                {
                   $self->{curInfo}->{genre} = $origtext;
                }
		else
		{
		   $self->{curInfo}->{genre} .= ", " . $origtext;
		}
		$self->{isTheme} = 1 ;
		return;
            }

	    # Capture of authors
	    elsif (($self->{isAuthor} == 1) && ($origtext ne '') && ($origtext =~ /^(?:(?!Ajax).)*$/))
            {
		# Lower case for author names, except for first letters
		$origtext =~ s/([[:alpha:]]+)/ucfirst(lc $1)/egi;
		$self->{author} = $origtext;
		$self->{isAuthor} = 2;
	    }

	    elsif ($self->{isAuthor} == 3 && $origtext ne '')
	    {
		if ($origtext =~ m/Traduction/i) {
		    if ($self->{curInfo}->{translator} eq '')
		    {
			$self->{curInfo}->{translator} = $self->{author};
		    }
		    else
		    {
			$self->{curInfo}->{translator} .= ", " . $self->{author};
		    }
		}
		else
		{
		    # affichage du rôle
		    if (! ($origtext =~ m/Auteur/i ))
		    {
			$self->{author} .= " ".$origtext;
			$self->{author} =~ s/, *$//;
		    }
		    if ($self->{curInfo}->{authors} eq '')
		    {
			$self->{curInfo}->{authors} = $self->{author};
		    }
		    else
		    {
			$self->{curInfo}->{authors} .= ", " . $self->{author};
		    }
		}
		$self->{isAuthor} = 0 ;
		return;
            }

	    # Capture of description
	    elsif (($self->{isDescription} == 2) && ($origtext ne ''))
	    {
                if ($self->{curInfo}->{description} eq '')
                {
                   $self->{curInfo}->{description} = $origtext;
                }
		else
		{
		   $self->{curInfo}->{description} .= " ".$origtext;
		}
                return ;
	    }
        }
    }


 
    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{hasField} = {
            title => 1,
            authors => 1,
            publication => 1,
            format => 0,
            edition => 0,
        };

	$self->{isUrl} = 0;
	$self->{isTitle} = 0;
	$self->{isPublication} = 0;
	$self->{isAuthor} = 0;
        $self->{isPage} = 0;
        $self->{isEditor} = 0;
        $self->{isISBN} = 0;
        $self->{isSerie} = 0;
        $self->{isSize} = 0;
        $self->{isDescription} = 0;
	$self->{isLanguage} = 0 ;
	$self->{isTheme} = 0 ;
	$self->{isPublication} = 0 ;
    
        $self->initTranslations();
        
        return $self;
    }
    
    sub getItemUrl
    {
	my ($self, $url) = @_;	
        return $url;
    }

    sub preProcess
    {
        my ($self, $html) = @_;

        # minimum initialization
        $self->{isUrl} = 0;
        $self->{isTitle} = 0;
        $self->{isPublication} = 0;
        $self->{isAuthor} = 0;

	if ($self->{parsingList})
        {
	    # Remove other commercial offers
	    $html =~ s|END SPONSORED LINKS SCRIPT.*||s;
	    # End of authors listing detection 
        }
        else
        {
	    $html =~ s|<BR[ /]*>|\n|gi;
	    $html =~ s|<I>||gi;
	    $html =~ s|</I>||gi;
            $html =~ s|\x{8C}|OE|gi;
            $html =~ s|\x{9C}|oe|gi;
            $html =~ s|&#146;|'|gi;
	}
        return $html;
    }

    sub getSearchUrl
    {
	my ($self, $word) = @_;
	return 'http://' . $self->baseWWWamazonUrl . '/s/ref=nb_sb_noss_1?url=search-alias=stripbooks&field-keywords=' . "$word";
    }
    
    sub baseWWWamazonUrl
    {   
	return "www.amazon.com";    
    }    

    sub getName
    {
        return "Amazon (US)";
    }
    
    sub getAuthor
    {
        return 'Varkolak - Kerenoc';
    }
    
    sub getLang
    {
        return 'EN';
    }

    sub getCharset
    {
        my $self = shift;
        return "ISO-8859-15";
    }
    
    sub getSearchFieldsArray
    {
        # TODO: Kerenoc: check if 'isbn' needs to be first to enable EAN/Scanner search
        return ['title', 'authors', 'isbn'];
    }

    sub getEanField
    {
        return 'isbn';
    }

    sub initTranslations
    {
        my $self = shift;
        
        $self->{translations} = {
            publisher     => "Publisher",
            language      => "Language",
            isbn          => "isbn",
            dimensions    => "Product Dimensions",
            series        => "Series",
            pages         => "pages",
            by            => "by"
        };
    }
}
1;
