package GCPlugins::GCfilms::GCAmazon;

###################################################
#
#  Copyright 2005-2010 Christian Jodar
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
#
###################################################

use strict;

use GCPlugins::GCfilms::GCfilmsAmazonCommon;

{
    package GCPlugins::GCfilms::GCPluginAmazon;

    use base qw(GCPlugins::GCfilms::GCfilmsAmazonPluginsBase);

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
        $self->{inside}->{$tagname}++;
        if ($self->{parsingEnded})
        {
            if ($self->{itemIdx} < 0)
            {
                $self->{itemIdx} = 0;
                $self->{itemsList}[0]->{url} = $self->{loadedUrl};
            }
            return;
        }

        if ($self->{parsingList})
        {
            if ($tagname eq 'ul')
            {
                $self->{beginParsing} = 1 if ($attr->{id} eq "s-results-list-atf");
            }
            return if ! $self->{beginParsing};
            if ($tagname eq 'h2')
            {
                $self->{isTitle} = 1;
            }
            elsif ($tagname eq 'publication')
            {
                $self->{isPublication} = 1;
            }
            elsif ($tagname eq 'actors')
            {
                $self->{isActors} = 1;
            }
            elsif ($tagname eq 'li')
            {
                $self->{beginParsing} = 2;
            }
            elsif ($tagname eq 'h5')
            {
                # Sponsored item
                $self->{beginParsing} = 1;
            }
            elsif ($tagname eq 'a' && $attr->{class} =~ /access-detail-page/ && $self->{beginParsing} eq 2)
            {
            	my $urlId;	
            		$self->{itemIdx}++;
            		$self->{itemsList}[$self->{itemIdx}]->{url} = $attr->{href};
            		$self->{itemsList}[$self->{itemIdx}]->{title} = $attr->{title};	
            		$self->{isTitle} = 0;	
                        $self->{beginParsing} = 1;
           }
        }
        else
        {
            if (($tagname eq "img") && $self->{insideImage} && (!$self->{curInfo}->{image}))
            {
                $self->{curInfo}->{image} = $attr->{src};
                $self->{insideImage} = 0;	
            }
            elsif (($tagname eq "img") && (!$self->{curInfo}->{image}) && ($attr->{id} eq "landingImage"))
            {
                $self->{curInfo}->{image} = $attr->{src};
                $self->{insideImage} = 0;	
            }
            elsif (($tagname eq 'table') && ($attr->{class} =~ m/product-details-meta/))
            {
                $self->{insideContent} = 1;
            }
            elsif (($tagname eq 'div') && ($attr->{id} =~ m/detail-bullets/))
            {
                $self->{insideContent} = 1;
            }
            elsif ($tagname eq 'div' && 
            		(   $attr->{class} eq 'productDescriptionWrapper'
            		 || $attr->{class} eq 'dv-simple-synopsis dv-extender'))
            {
                	$self->{insideSynopsis} = 1
                    	if (!$self->{curInfo}->{synopsis});	
            }
            elsif (($tagname eq 'div') && ($attr->{class} eq 'emptyClear'))
            {
                $self->{insideSynopsis} = 0;
            }
            elsif (($tagname eq "span") && ($self->{insideAge}) && ($attr->{class} =~ /medSprite/))
            {
                $attr->{class} =~ s/\s*$//;
                $self->{curInfo}->{age} = 2 if ($attr->{class} =~ m/G$/);
                $self->{curInfo}->{age} = 5 if ($attr->{class} =~ m/PG$/);
                $self->{curInfo}->{age} = 13 if ($attr->{class} =~ m/PG13$/);
                $self->{curInfo}->{age} = 18 if ($attr->{class} =~ m/R$/);
                $self->{insideAge} = 0;
            }
            elsif ($tagname eq "span" && $attr->{id} eq "productTitle")
            {
            	$self->{insideName} = 1;
            }
            elsif ($tagname eq "h1" && $attr->{'data-automation-id'} eq "title")
            {
            	$self->{insideName} = 1;
            }
            elsif ($tagname eq "h1" && $attr->{id} eq "aiv-content-title")
            {
                $self->{insideName} = 1;
            }
            elsif ($tagname eq "span" && $attr->{class} eq "release-year")
            {
            	$self->{insideDate} = 1;
            }
            elsif ($tagname eq "div" && $attr->{class} eq "dp-meta-icon-container")
            {
            	$self->{insideImage} = 1;
            }
            elsif ($tagname eq "dt") 
            {
            	$self->{insideTime} = 2;
            }          
        }
    }

    sub end
    {
        my ($self, $tagname) = @_;
        $self->{inside}->{$tagname}--;
        if ($self->{insideContent} && ($tagname eq "li" || $tagname eq "td"))
        {
            $self->{insideActors} = 0;
            $self->{insideDirector} = 0;
            $self->{insideGenre} = 0;
        } 
        elsif ($self->{insideContent} && ($tagname eq "table"))
        {
            $self->{insideContent} = 0;
        }
        elsif ($self->{insideSynopsis} && ($tagname eq "div"))
        {
            $self->{insideSynopsis} = 0;
        }
    }

    sub text
    {
        my ($self, $origtext) = @_;
        return if GCPlugins::GCstar::GCPluginAmazonCommon::text(@_);
        return if length($origtext) < 2;
        return if ($self->{parsingEnded});
        if ($self->{parsingList})
        {
            return if $self->{beginParsing} ne 2;
            if ($self->{isPublication})
            {
                $origtext =~ m/([0-9]{4})/;
                $self->{itemsList}[$self->{itemIdx}]->{date} = $1;
                $self->{isPublication} = 0;
                return;
            }
            elsif ($self->{isActors})
            {
                $origtext =~ s/^\s*//;
                $origtext =~ s/\s*$//;
                $self->{itemsList}[$self->{itemIdx}]->{actors} = $origtext
                    if ! $self->{itemsList}[$self->{itemIdx}]->{actors};
                $self->{isActors} = 0;
                return;
            }
        }
       	else
        {
            $origtext =~ s/^\s*//;
            $origtext =~ s/\s$//;
            $origtext =~ s/\s{2,}/ /g;
            $origtext =~ s/^,$//;
			
            if ($self->{insideName})
            { 
            	if ($origtext ne '')
            	{
                	$self->{curInfo}->{title} = $origtext;
                        $self->{insideName} = 0;
            	}
            }
            elsif ($self->{insideDate})
            {
            	$self->{curInfo}->{date} = $origtext;
            	$self->{insideDate} = 0;
            }
            elsif (($self->{insideActors}) && ($origtext ne ''))
            {
                if ($self->{actorsCounter} < $GCPlugins::GCfilms::GCfilmsCommon::MAX_ACTORS)
                {
                    push @{$self->{curInfo}->{actors}}, [$origtext];                
                    $self->{actorsCounter}++;
                }
            }
            elsif (($self->{insideDirector}) && ($origtext ne ''))
            {
                $self->{curInfo}->{director} .= ", "
                    if $self->{curInfo}->{director};
                $self->{curInfo}->{director} .= $origtext;
            }
            elsif ($self->{insideTime} eq 2 && ($origtext ne ''))
            {
                $self->{insideTime} = 1 if ($origtext =~ /Runtime:/);
            }
            elsif ($self->{insideTime} eq 1 && ($origtext ne ''))
            {
            		if ($origtext =~ m/([0-9]*) *hours *([0-9]*) *minutes/)
            		{
            			$origtext = $1 * 60 + $2 ;
            			$origtext .= " minutes";
            		}
            	
                	$self->{curInfo}->{time} = $origtext;
                	$self->{insideTime} = 0;
            }
            elsif ($self->{insideGenre} && $origtext ne '')
            {
            	$self->{curInfo}->{genre} .= ", "
                    if $self->{curInfo}->{genre};
                $self->{curInfo}->{genre} .= $origtext;
            }
            elsif (($self->{insideSynopsis}) && ($origtext ne ''))
            {
                $self->{curInfo}->{synopsis} .= $origtext;
            }
            elsif ($self->{insideContent} && ($origtext ne ''))
            {
            	# Amazon Movies and TV
                $self->{insideActors} = 1 if $origtext =~ /Actors:/;
                $self->{insideDirector} = 1 if $origtext =~ /Directors:/;
                $self->{insideTime} = 1 if $origtext =~ /Run Time:/;
                $self->{insideGenre} = 1 if $origtext =~ /Genre:/;
                $self->{insideAge} = 1 if $origtext =~ /Rating:/;
                $self->{insideDate} = 1 if $origtext =~ /Release Date/;
                $self->{insideRegion} = 1 if $origtext =~ /Region/;
                # Amazon Videos
                $self->{insideDirector} = 1 if $origtext =~ /Director/;
                $self->{insideGenre} = 1 if $origtext =~ /Genres/;
                $self->{insideActors} = 1 if $origtext =~ /Starring/;
                $self->{insideActors} = 1 if $origtext =~ /Supporting actors/;                                
            }  
        }
    } 

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);
        $self->{hasField} = {
            title => 1,
            date => 1,
            director => 0,
            actors => 1,
        };

        $self->{suffix} = 'com';

        $self->{curName} = undef;
        $self->{curUrl} = undef;
        $self->{insideName} = 0;
        $self->{insideDate} = 0;
        $self->{insideActor} = 0;
        $self->{insideDirector} = 0;
        $self->{insideTime} = 0;
        $self->{insideGenre} = 0;
        $self->{insideAge} = 0;
        $self->{insideImage} = 0;
        $self->{insideRegion} = 0;

        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;
        $html = $self->SUPER::preProcess($html);
        if ($self->{parsingList})
        {
            $self->{isItem} = 0;
            $html =~ s|~(.*?)<span class="bindingBlock">\(<span class="binding">(.*?)</span>( - .*?[0-9]{4})?\)</span>|<actors>$1</actors><format>$2</format><publication>$3</publication>|gsm;
        }
        else
        {
            $html =~ s/(<i>|<\/i>)//gim;
            $html =~ s/<p>/\n/gim;
            $html =~ s|</p>|\n|gim;
            $html =~ s/(<ul>|<\/ul>)/\n/gim;
            $html =~ s/<li>([^<])/- $1/gim;
            $html =~ s|([^>])</li>|$1\n|gim;
            $html =~ s|<br ?/?>|\n|gi;
            $html =~ s|<a href="/gp/imdb/[^"]*">(.*?)</a>|$1|gm;
            $html =~ s|<a href="[^"]*search-alias=dvd&field-keywords=[^"]*">(.*?)</a>|$1|gm;
        }
        $self->{parsingEnded} = 0;
        $self->{alreadyRetrieved} = {};
        $self->{beginParsing} = 0;

        return $html;
    }

    sub getName
    {
        return "Amazon (US)";
    }
    
    sub getLang
    {
        return 'EN';
    }

}

1;
