package GCPlugins::GCfilms::GCAmazonUK;

###################################################
#
#  Copyright 2005-2010 Christian Jodar
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
#
###################################################

use strict;

use GCPlugins::GCfilms::GCfilmsAmazonCommon;

{
    package GCPlugins::GCfilms::GCPluginAmazonUK;

    use base qw(GCPlugins::GCfilms::GCfilmsAmazonPluginsBase);

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
    
        $self->{inside}->{$tagname}++;

        if ($self->{parsingEnded})
        {
            if ($self->{itemIdx} < 0)
            {
                $self->{itemIdx} = 0;
                $self->{itemsList}[0]->{url} = $self->{loadedUrl};
            }
            return;
        }

        if ($self->{parsingList})
        {
            $self->{beginParsing} = 1
                if ($tagname eq 'ul' && $attr->{id} eq "s-results-list-atf");
                
            return if ! $self->{beginParsing};
            
            if ($tagname eq 'span' && $attr->{class} eq "a-size-small a-color-secondary")
            {
                $self->{isPublication} = 1 
                    if ($self->{itemIdx} && !$self->{itemsList}[$self->{itemIdx}]->{date});
                $self->{isActors} = 1 
                    if ($self->{itemIdx} && !$self->{itemsList}[$self->{itemIdx}]->{actors});
            }           
            elsif ($tagname eq 'a' && $attr->{class} =~ /s-access-detail-page/)
            {
                $self->{itemIdx}++;
                $self->{itemsList}[$self->{itemIdx}]->{url} = $attr->{href};
                $self->{itemsList}[$self->{itemIdx}]->{title} = $attr->{title};   
            }
        }
        else
        {
            if (($tagname eq "img") && ($attr->{id} eq 'landingImage') && (!$self->{curInfo}->{image}))
            {
		if ($attr->{src} =~ m/data:image/)
		{
		    # image sent in HTML (film 2010)
		    $self->{curInfo}->{image} = $attr->{"data-a-dynamic-image"} if ($attr->{"data-a-dynamic-image"});
		}
		$self->{curInfo}->{image} = $attr->{src} if (! ($attr->{src} =~ m/data:image/));
            }
            elsif (($tagname eq 'div') && ($attr->{class} eq 'content'))
            {
                $self->{insideContent} = 1;
            }
            elsif ($tagname eq 'h3')
            {
                $self->{insideSynopsis} = 1
                    if (!$self->{curInfo}->{synopsis});
                $self->{insideOriginal} = 1
                    if (!$self->{curInfo}->{original});
            }
	    elsif (($tagname eq 'br' || $tagname eq 'a') && $self->{insideSynopsis} eq 2)
	    {
		$self->{insideSynopsis} = 0;
	    }
            elsif (($tagname eq "span") && ($self->{insideAge}) && ($attr->{class} =~ /medSprite/))
            {
                $attr->{class} =~ s/\s*$//;
                $self->{curInfo}->{age} = 2 if ($attr->{class} =~ m/G$/);
                $self->{curInfo}->{age} = 5 if ($attr->{class} =~ m/PG$/);
                $self->{curInfo}->{age} = 13 if ($attr->{class} =~ m/PG13$/);
                $self->{curInfo}->{age} = 18 if ($attr->{class} =~ m/R$/);
                $self->{insideAge} = 0;
            }
            elsif ($tagname eq "span")
            {
		$self->{insideTheme} = 1 if ($attr->{class} eq "zg_hrsr_ladder");
		$self->{isTitle} = 1 if ($attr->{id} eq 'productTitle');
            }
	}
    }

    sub end
    {
        my ($self, $tagname) = @_;
	
        $self->{inside}->{$tagname}--;
        if ($tagname eq "li")
        {
            $self->{insideActors} = 0;
            $self->{insideDirector} = 0;
        }
	elsif ($tagname eq 'div' && $self->{insideSynopsis} eq 2)
	{
	    $self->{insideSynopsis} = 0;
	}
	elsif ($tagname eq 'span')
	{
	    $self->{insideTheme} = 0;
	}
    }

    sub text
    {
        my ($self, $origtext) = @_;

        return if length($origtext) < 2;
           
        if ($self->{parsingList})
        {
            return if ! $self->{beginParsing};
            if (($self->{inside}->{title})
             && ($origtext !~ /^Amazon.co.uk/))
            {
                $self->{parsingEnded} = 1;
            }
            elsif ($self->{isPublication} eq 1 && $origtext =~ m/^[0-9]{4}$/)
            {
                $origtext =~ m/([0-9]{4})/;
                $self->{itemsList}[$self->{itemIdx}]->{date} = $1;
                $self->{isPublication} = 0;
                return;
            }
            elsif ($self->{isActors} eq 1) 
	    {
		$self->{isActors} = 2 if ($origtext =~ m/Starring/);
	    }
            elsif ($self->{isActors} eq 2)
            {
                $origtext =~ s/^\s*//;
                $origtext =~ s/\s*$//;
                $self->{itemsList}[$self->{itemIdx}]->{actors} = $origtext
                    if ! $self->{itemsList}[$self->{itemIdx}]->{actors};
                $self->{isActors} = 0;
                return;
            }
        }
        else
        {
            $origtext =~ s/\s{2,}/ /g;

            if (($self->{insideActors}) && ($origtext !~ /^,/))
            {
                $origtext =~ s/^\s//;
                $origtext =~ s/\s+,/,/;
                if ($self->{actorsCounter} < $GCPlugins::GCfilms::GCfilmsCommon::MAX_ACTORS)
                {
                    push @{$self->{curInfo}->{actors}}, [$origtext];
                    $self->{actorsCounter}++;
                }
            }
            elsif (($self->{insideDirector}) && ($origtext !~ /^,/))
            {
                $origtext =~ s/^\s//;
                $origtext =~ s/,.$//;
                $self->{curInfo}->{director} .= ", "
                    if $self->{curInfo}->{director};
                $self->{curInfo}->{director} .= $origtext;
            }
            elsif ($self->{insideTime})
            {
                $origtext =~ s/^\s//;
                $origtext =~ s/\n//g;
	        $self->{curInfo}->{time} = $origtext;
	        $self->{curInfo}->{time} =~ s/ [a-z]+/ /i;
	     
                $self->{insideTime} = 0;
            }
            elsif ($self->{insideDate})
            {
                $origtext =~ s/^\s//;
                $origtext =~ s/\n//g;
	        $origtext =~ s/\-$//;	     
                $self->{curInfo}->{date} = $self->decodeDate($origtext);
                $self->{insideDate} = 0;
            }
            elsif (($self->{insideOriginal} eq 1) && ($origtext =~ m/Product details/i ))
            {
                $self->{insideOriginal} = 2;
            }
            elsif (($self->{insideOriginal} eq 2) && ($origtext ne ''))
            {
		if ($origtext =~ m/\(.*\),.*,.*minutes/)
		{
		    $origtext =~ s/.*\(//;
		    $origtext =~ s/\).*//;		    
		    $self->{curInfo}->{original} .= $origtext;
		}
		$self->{insideOriginal} = 0;
            }
            elsif (($self->{insideSynopsis} eq 1) && ($origtext eq 'Synopsis' || $origtext eq 'Amazon.co.uk'))
            {
                $self->{insideSynopsis} = 2;
            }
            elsif (($self->{insideSynopsis} eq 2) && ($origtext ne ''))
            {
                $self->{curInfo}->{synopsis} .= $origtext;
            }
            elsif ($self->{insideAudio})
            {
                $origtext =~ s/^\s//;
                $self->{curInfo}->{audio} = $origtext;
                $self->{insideAudio} = 0;
            }
            elsif ($self->{insideSubTitle})
            {
                $origtext =~ s/^\s//;
                $self->{curInfo}->{subt} = $origtext;
                $self->{insideSubTitle} = 0;
            }
	    elsif ($self->{insideTheme} eq 1)
	    {
		if (! ($origtext =~ m/Blu-ray/i || $origtext =~ m/[>,]/ || $origtext =~ m/^in/))
		{
		    $origtext = ", ".$origtext if ($self->{curInfo}->{genre});
		    $self->{curInfo}->{genre} .= $origtext;
		}
	    }
            elsif ($self->{inside}->{b})
            {
                $self->{insideActors} = 1 if $origtext =~ /Actors:/;
                $self->{insideDirector} = 1 if $origtext =~ /Directors.*:/;
                $self->{insideDate} = 1 if $origtext =~ /Release date/i;
                $self->{insideTime} = 1 if $origtext =~ /Run Time/i;
                $self->{insideAudio} = 1 if $origtext =~ /Language/;
                $self->{insideSubTitle} = 1 if $origtext =~ /Subtitles/;
            }        
	    elsif ($self->{isTitle} eq 1 && $origtext =~ m/[a-z0-9]/i)
	    {
		$origtext =~ s/^ *//;
		$self->{curInfo}->{title} = $origtext;
		$self->{isTitle} = 0;
	    }
        }
    } 

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{hasField} = {
            title => 1,
            date => 1,
            director => 0,
            actors => 1,
        };

        $self->{suffix} = 'co.uk';


        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;

        $html = $self->SUPER::preProcess($html);
        if (!$self->{parsingList})
        {
	    # problem when an image is embedded in the HTML (film 2010)
	    # the attribute data-a-dynamic-image="{&quote;http would return {
	    $html =~ s/data-a-dynamic-image="\{&quot;/data-a-dynamic-image="/;
        }

        $self->{parsingEnded} = 0;
        $self->{currentRetrieved} = '';
        $self->{beginParsing} = 0;

        return $html;
    }
    
    sub getName
    {
        return "Amazon (UK)";
    }

    sub getLang
    {
        return 'EN';
    }

    sub decodeDate
    {
	my ($self, $date) = @_;

	# date déjà dans le bon format
	return $date if ($date =~ m|/.*/|);
	# année seule
	return "01/01/".$date if ($date =~ m/^[0-9]+$/);
	# pas d'année
	return '' if (! $date =~ m/[0-9][0-9][0-9][0-9]/);
	if ($date =~ m/trimestre/)
	{
	    $date =~ s/[^0-9]* *trimestre */ /;
	    my @dateItems = split(/\s/, $date);
	    $date = "01/01/".$dateItems[1] if ($dateItems[0] eq '1');
	    $date = "01/04/".$dateItems[1] if ($dateItems[0] eq '2');
	    $date = "01/07/".$dateItems[1] if ($dateItems[0] eq '3');
	    $date = "01/10/".$dateItems[1] if ($dateItems[0] eq '4');
	    return $date;
	}
        # replacement par DateTimeFormatStrToTime si merge GCUtils.pm accepté 
    
	# date à convertir au format jour/mois/année
	my @dateItems = split(/\s/, $date);
	my @listeMois = ("janvier","février","mars","avril","mai","juin",
			 "juillet","août","septembre","octobre","novembre","décembre");
	my $mois = 0;
	while ($mois < (scalar @listeMois) && $dateItems[1] ne $listeMois[$mois])
	{
	    $mois++;
	}
	$mois++;
	return $dateItems[0]."/".sprintf("%02d",$mois)."/".$dateItems[2];
    }

}

1;
